HMM<-function(allecounts, dist, reps=1, epsilon0=0.075,alpha10=2,beta010=2,alpha20=2,beta020=2,maxiter=20,cnv=NULL,estimate.alpha=TRUE,common.alpha=FALSE, dist.dep=FALSE)
{
  # Arguments:
  # allecounts: 3 dimentional data matrix with RNA read counts [SNP, Allele, Animal].
  # reps: number of replicates. reps=1 for real data set.
  # epsilon0: error rate. Between 0 and 1
  # alpha010, beta010: starting values for beta distribution parameters, for ASE-Low rate.
  # alpha020, beta020: starting values for beta distribution parameters, for ASE-High rate.
  # Values:
  # allecounts: 3 dimentional data matrix with RNA read counts [SNP, Allele, Animal].
  # GenoEsti: 3 dimentional matrix with genotype prediction result [SNP, Animal, replicate].
  #           For real data, #replicate=1, so the dimention is [#SNPs, #Animals, 1].
  # Parameters: Estimation of paramter values after convergence with length 30.
  #             c(epsilon0,alpha10,beta010,alpha20,beta020,as.vector(Amat))
  #             Amat is a 5 by 5 transition matrix.
  
  T<-dim(allecounts)[1]
  n<-dim(allecounts)[3]
  neg.loc<-which(allecounts<0,arr.ind=TRUE)
  if (nrow(neg.loc)>0)
  { warning ("Some counts are less than 0 and the values were changed to be 0") 
    allecounts[neg.loc]=0}
  GenotypeEsti<-array(0,c(T,n,reps))
  Parameters<-NULL
  deltas<-array(0,c(T,n,reps))
  if(is.null(cnv)){
    cnv<-T*n*0.01
  } else {
    cnv<-T*n*cnv
  }
  
  MacMin<-.Machine$double.xmin
  
  for (W in 1:reps)
  {
    M<-5
    Amat<-array(1/M,c(M,M,T-1)) # This is the default opt, dist.dep==FALSE
    if (dist.dep==TRUE)
    {
      a_mat<-matrix(1/M, M, M)
      rho<-10
      Amat<-Amat_fun(a_mat, rho=rho, dist, T, M) 
    }
    
    maxlocation<-matrix(0,T,n)
    ObsLikelihood<-0
    repcounts<-1
    converge<-TRUE
    
    repeat
    {
      if(T<2)
      {
        warning ("Too few SNP")
        converge<-F
        break
      }
      
      ## step 1: computing eln(alpha), eln(beta), and eln(Px)
      
      pi0<-rep(1/M,M)
      
      ## compute eln(alpha)
      
      lnalphas<-array(0,c(T,M,n))
      Logfxmat<-Logfx(c(1:M),allecounts,epsilon0,alpha10,beta010,alpha20,beta020, MacMin)
      
      for (k in 1:n)
      {
        for (i in 1:M)
        {
          lnalphas[1,i,k]<-elnproduct(eln(pi0[i]),Logfxmat[1,k,i])
        }
        for (t in 2:T)
        {
          for (j in 1:M)
          { logalpha<--9999
            for (i in 1:M)
            {
              logalpha<-elnsum(logalpha,elnproduct(lnalphas[t-1,i,k],eln(Amat[i,j,t-1])))
            }
            lnalphas[t,j,k]<-elnproduct(logalpha,Logfxmat[t,k,j])
          }
        }
      }
      
      
      ## compute eln(beta)
      
      lnbetas<-array(0,c(T,M,n))
      for (k in 1:n)
      {
        for (t in (T-1):1)
        {
          for (i in 1:M)
          { logbeta<--9999
            for (j in 1:M)
            {logbeta<-elnsum(logbeta,elnproduct(eln(Amat[i,j, t]), elnproduct(Logfxmat[t+1,k,j],lnbetas[t+1,j,k])))}
            lnbetas[t,i,k]<-logbeta
          }
        }
      } 
      
      ## compute Lk in log-space
      ## Lk is the posterior probability of the state given the observations.
      
      lnLk<-Lk<-array(0,c(T,M,n))
      for (k in 1:n)
      {
        for (t in 1:T)
        {
          normalizer<--9999
          for (i in 1:M)
          {lnLk[t,i,k]<-elnproduct(lnalphas[t,i,k],lnbetas[t,i,k])
           normalizer<-elnsum(normalizer,lnLk[t,i,k])
          }
          for (i in 1:M)
          {lnLk[t,i,k]<-elnratio(lnLk[t,i,k],normalizer)
           Lk[t,i,k]<-eexp(lnLk[t,i,k])}
        }
      }
      
      ## compute Hklt in log-space"
      
      lnHklt<-Hklt<-array(0,c(T,M,M,n))
      
      for (k in 1:n)
      {
        for (t in 1:(T-1))
        {
          normalizer<--9999
          for (i in 1:M)
          { 
            for (j in 1:M)
            { lnHklt[t,i,j,k]<-elnproduct(lnalphas[t,i,k],elnproduct(eln(Amat[i,j, t]),elnproduct(Logfxmat[t+1,k,j],lnbetas[t+1,j,k])))
              normalizer<-elnsum(normalizer,lnHklt[t,i,j,k])
            }
          }
          for (i in 1:M)
          { for (j in 1:M)
          {lnHklt[t,i,j,k]<-elnratio(lnHklt[t,i,j,k],normalizer)
           Hklt[t,i,j,k]<-eexp(lnHklt[t,i,j,k])
          }
          }
        }
      }
      
      ## Update parameters akl, delta, epsilon and omega
      if (dist.dep==FALSE)
      {
        nAmat<-array(0,c(M,M, T-1)) 
        for (i in 1:M)
          for (j in 1:M)
          {
            numerator<--9999
            denominator<--9999
            for (k in 1:n)
            {
              for (t in 1:(T-1))
              {numerator<-elnsum(numerator,lnHklt[t,i,j,k])
               denominator<-elnsum(denominator,lnLk[t,i,k])
              }
            } 
            nAmat[i,j,]<-eexp(elnratio(numerator,denominator))
          }
      }
      
      if (dist.dep==TRUE)
      {
        a_find_mat_func<-function(hx, dist, i, rho, Hklt)
        {            
          Hklt_i <- Hklt[,i,,]
          Hklt_i_pigsum <- apply(Hklt_i,c(1,2),sum)
          #Hklt_i_0<-Hklt_i
          #Hklt_i_0[,,]<-0
          Hklt_k_timesum<-apply(Hklt_i,2,sum)
          trans_Hklt_k<-matrix(rep(0,M*(T-1)),nrow=(T-1))
          for (t in 1:(T-1))
          {
            trans_Hklt_k_d <- Hklt_k_timesum*(1-exp(-dist[t]*rho))/hx
            trans_Hklt_k[t,] <- trans_Hklt_k_d
            trans_Hklt_k[t,i] <- 1-sum(trans_Hklt_k_d)
          }
          return(sum(delete0product(Hklt_i_pigsum[1:(T-1),],log(trans_Hklt_k))))
        }
        
        
        a_find_llimit_func<-function(dist, i, rho, Hklt)
        {
          # This function is to give a lower limit of the searching range of x in using uniroot of
          # function p_find_mat_func. For each i, first run this function, then using uniroot on 
          # function p_find_mat_func for the root x.
          
          Hklt_noi <- Hklt[,i,-i,] # Hklt_noi is an array of (T-1)*(M-1)*n for a fixed i (fix the second dimension).
          M11<-apply(Hklt_noi,2,sum)
          M1<-sum(Hklt[,i,,])
          #M21<-M1*(1-exp(-(posi_vec[2:T]-posi_vec[1:(T-1)])^r))
          max_M111<-max(M1,M11)
          
          return(max_M111+min(max_M111,1)*1e-10)
        }
        
        
        r_find_func <- function(rho, a_mat, dist, Hklt, T)
        {
          # This function is part of the conditional expectation function in E-M, and it is used 
          # to search an optimal value of r.
          #
          
          trans_mat_r <- Amat_fun(a_mat, rho, dist, T, M)
          trans_mat_r<-wrap(trans_mat_r,map=list(3,1,2))
          if (T==2)
          {MATRIX <- delete0product(apply(Hklt[1,,,], c(1,2), sum),log(trans_mat_r[1,,]))}
          if (T>2)
          {MATRIX <- delete0product(apply(Hklt[1:(T-1), , , ], c(1,2,3), sum),log(trans_mat_r))}
          return(sum(MATRIX))
        }
        
        a_mat_ud<-matrix(c(rep(0,M*M)),nrow=M)
        for (k in 1: M)
        {
          llimit <- a_find_llimit_func(dist, k, rho, Hklt)
          xvalvec<-matrix(seq(llimit,llimit+5000,length=1000),500,1)
          yvec<-apply(xvalvec,1,a_find_mat_func,dist=dist, i=k, rho=rho, Hklt=Hklt)
          x_vec<-xvalvec[which.max(yvec)]
          # x_vec <- optimize(a_find_mat_func, c(llimit,llimit+500), maximum=TRUE, tol = 0.001, dist=dist, i=i, rho=rho, Hklt=Hklt)[[1]]
          a_mat_ud[k,] <- apply(Hklt[,k,,],2,sum)/x_vec  
          a_mat_ud[k,k] <- 0              
        }	 
        
        
        rho_ud<-optimize(r_find_func, c(0, 1), maximum=TRUE, tol = 0.001, a_mat=a_mat_ud, dist=dist, Hklt=Hklt, T=T)[[1]] 
        
        # eps_p_r_pi <-max(abs(a_mat_ud - a_mat), abs(rho_ud-rho))
        a_mat <- a_mat_ud # pudate p-mat in the transition matrix
        rho <- rho_ud # update r in the transition matrix  
        
        
        nAmat<-Amat_fun(a_mat, rho, dist, T, M)
        
      }
      
      
      ## Observed likelihood functions
      
      ObsLike<-function(epsilon,alpha1,alpha2,Lk,Hklt,nAmat,allecounts,M,pi0, MacMin)
      {
        beta01<-alpha1
        beta02<-alpha2
        Logfxmat<-Logfx(c(1:M),allecounts,epsilon,alpha1,beta01,alpha2,beta02, MacMin)
        Logfxmat0<-wrap(Logfxmat, map=list(1, 3, 2))
        nAmat0<-wrap(nAmat,map=list(3,1,2))
        class(Logfxmat0)<-"numeric"
        if (T>2)
        {nObsLike<-sum(Lk[1,,]*log(matrix(rep(pi0,n),M,n,byrow=T)))+sum(delete0product(apply(Hklt[-T,,,],c(1,2,3),sum),log(nAmat0)))+sum(Lk*Logfxmat0)}
        if (T==2)
        {nObsLike<-sum(Lk[1,,]*log(matrix(rep(pi0,n),M,n,byrow=T)))+sum(delete0product(apply(Hklt[1,,,],c(1,2),sum),log(nAmat0[1,,])))+sum(Lk*Logfxmat0)}   
        return(nObsLike)
      }
      ObsLike4alpha1<-function(alpha1,alpha2,epsilon,Lk,Hklt,nAmat,allecounts,M,pi0, MacMin)
      {
        OL<-ObsLike(epsilon,alpha1,alpha2,Lk,Hklt,nAmat,allecounts,M,pi0, MacMin)
        return(OL)
      }
      ObsLike4alpha2<-function(alpha2,alpha1,epsilon,Lk,Hklt,nAmat,allecounts,M,pi0, MacMin)
      {
        OL<-ObsLike(epsilon,alpha1,alpha2,Lk,Hklt,nAmat,allecounts,M,pi0, MacMin)
        return(OL)
      }
      ObsLike4alphabeta<-function(alpha1,epsilon,Lk,Hklt,nAmat,allecounts,M,pi0, MacMin)
      {
        alpha2<-alpha1
        OL<-ObsLike(epsilon,alpha1,alpha2,Lk,Hklt,nAmat,allecounts,M,pi0, MacMin)
        return(OL)
      }
      
      Maxeps<-optimize(ObsLike,interval=c(0.0001,0.7),alpha1=alpha10,alpha2=alpha20,Lk=Lk,Hklt=Hklt,nAmat=nAmat,allecounts=allecounts,M=M,pi0=pi0,maximum=TRUE, MacMin=MacMin)
      nepsilon<-Maxeps$maximum
      
      if(estimate.alpha){
        if (common.alpha){
          Maxalphabeta<-optimize(ObsLike4alphabeta, interval=c(2,10),epsilon=nepsilon,Lk=Lk,Hklt=Hklt,nAmat=nAmat,allecounts=allecounts,M=M,pi0=pi0,maximum=TRUE, MacMin=MacMin)
          nalpha1<-Maxalphabeta$maximum
          nalpha2<-nalpha1
          nObsLikelihood<-Maxalphabeta$objective
        } else {
          Maxalpha1<-optimize(ObsLike4alpha1,c(2,10),alpha2=alpha20,epsilon=nepsilon,Lk=Lk,Hklt=Hklt,nAmat=nAmat,allecounts=allecounts,M=M,pi0=pi0,maximum=TRUE, MacMin=MacMin)
          nalpha1<-Maxalpha1$maximum
          Maxalpha2<-optimize(ObsLike4alpha2,c(2,10),alpha1=nalpha1,epsilon=nepsilon,Lk=Lk,Hklt=Hklt,nAmat=nAmat,allecounts=allecounts,M=M,pi0=pi0,maximum=TRUE, MacMin=MacMin)
          nalpha2<-Maxalpha2$maximum
          nObsLikelihood<-Maxalpha2$objective
        }} else {
          nalpha1<-alpha10
          nalpha2<-alpha20
          nObsLikelihood<-Maxeps$objective
        }
      Lkorder<-apply(Lk,c(1,3),order)
      nmaxlocation<-Lkorder[5,,]
      
      cond<-(abs(nObsLikelihood-ObsLikelihood)<1e-3)
      if (!is.na(cond)){
        if ((sum(nmaxlocation!=maxlocation)<cnv)||(repcounts>maxiter)||(abs(nObsLikelihood-ObsLikelihood)<1e-3))
        {
          if (repcounts>maxiter) {
            warning("Too many iterations. Not converge!")
            converge<-F
          }
          
          if (sum(nmaxlocation!=maxlocation)<cnv) cat("States not change. Converges.","\n")
          if (abs(nObsLikelihood-ObsLikelihood)<1e-3) cat("Likelihood not change. Converges.","\n")
          break
        }} else {
          warning("A mat contains zeroes try changing chunk length")
          converge<-F
          break
        }
      
      cat(repcounts,"iteration","\n") 
      cat(sum(nmaxlocation!=maxlocation),"SNP genotype predictions are changed with observed log-likelihood", nObsLikelihood,"\n")
      
      Amat<-nAmat
      epsilon0<-nepsilon
      alpha10<-nalpha1
      beta010<-nalpha1
      alpha20<-nalpha2
      beta020<-nalpha2
      maxlocation<-nmaxlocation
      ObsLikelihood<-nObsLikelihood
      repcounts<-repcounts+1	
    }
    GenotypeEsti[,,W]<-maxlocation
    if (dist.dep==FALSE)
    {Parameters<-rbind(Parameters,c(epsilon0,alpha10,beta010,alpha20,beta020,as.vector(Amat)))}
    if (dist.dep==TRUE)
    {Parameters<-rbind(Parameters,c(epsilon0,alpha10,beta010,alpha20,beta020,rho,as.vector(a_mat)))}
  }
  
  ## functions for computing posterior pdf
  ## delta is the place where we are going to evaluate the posterior density
  ## allecounts is a T*2*n array from each replicate
  ## genetypes is the estimated genotypes for all the replicates
  ## epsilon, alpha and beta0 are estimated value of epsilon, alpha and beta.
  
  postdelta<-function(delta,allecounts,posat,epsilon,alpha10,alpha20,beta010,beta020,HL)
  {
    if (!is.null(dim(posat)))
    {Ngenes<-dim(posat)[1]}
    if (is.null(dim(posat)))
    {Ngenes<-length(posat)/2}
    posat<-matrix(t(posat),Ngenes,2,byrow=TRUE)
    prob0<-(1-2*epsilon)*delta+epsilon
    alles<-matrix(0,Ngenes,2)
    for (i in 1:Ngenes){
      alles[i,]<-allecounts[posat[i,1],,posat[i,2]]}
    if (HL=="H")
    {
      posat3<-cbind(posat,3)
      fxval3<-apply(matrix(Logfxmat[posat3],Ngenes,1),1,eexp)
      fxdelta<-dbinom(alles[,1],prob=prob0,size=rowSums(alles))*dbeta((delta-0.5)/0.5,alpha10,beta010)
      postden<-fxdelta/fxval3
    }
    if (HL=="L")
    {
      posat4<-cbind(posat,4)
      fxval4<-apply(matrix(Logfxmat[posat4],Ngenes,1),1,eexp)
      fxdelta<-dbinom(alles[,1],prob=prob0,size=rowSums(alles))*dbeta((0.5-delta)/0.5,alpha20,beta020)
      postden<-fxdelta/fxval4
    }
    return(postden)
  }
  
  postden3<-postden4<-postinf<-NULL
  delta3<-matrix(seq(0.5,1,length=100),100,1)
  positions3<-which(GenotypeEsti==3,arr.ind=TRUE)
  posat3<-positions3[,1:2]
  if (length(posat3)!=0)
  {   
    fpost3<-apply(delta3,1,postdelta,allecounts=allecounts,posat=posat3,epsilon=epsilon0,alpha10=alpha10,beta010=beta010,alpha20=alpha20,beta020=beta020,HL="H")
    if (!is.null(dim(fpost3)))
    {post3mode<-delta3[apply(fpost3,1,which.max)]
     postinf3<-cbind(posat3,'ASE-H',post3mode)
     postden3<-cbind(posat3,fpost3)
     colnames(postden3)<-c("Locus of SNP", "ID",as.character(delta3))
     colnames(postinf3)<-c("Locus of SNP", "ID","ASE Status","ASE ratio estimate")}
    if (is.null(dim(fpost3)))
    {post3mode<-delta3[which.max(fpost3)]
     postinf3<-c(posat3,'ASE-H',post3mode)
     postden3<-c(posat3,fpost3)
     names(postden3)<-c("Locus of SNP", "ID",as.character(delta3))
     names(postinf3)<-c("Locus of SNP", "ID","ASE Status","ASE ratio estimate")}
    postinf<-rbind(postinf,postinf3)
  }
  
  delta4<-matrix(seq(0,0.5,length=100),100,1)
  positions4<-which(GenotypeEsti==4,arr.ind=TRUE)
  posat4<-positions4[,1:2]
  if (length(posat4)!=0)
  {  
    fpost4<-apply(delta4,1,postdelta,allecounts=allecounts,posat=posat4,epsilon=epsilon0,alpha10=alpha10,beta010=beta010,alpha20=alpha20,beta020=beta020,HL="L")
    if (!is.null(dim(fpost4)))
    {post4mode<-delta4[apply(fpost4,1,which.max)]
     postinf4<-cbind(posat4,'ASE-L',post4mode)
     postden4<-cbind(posat4,fpost4)
     colnames(postden4)<-c("Locus of SNP", "ID",as.character(delta4))
     colnames(postinf4)<-c("Locus of SNP", "ID","ASE Status","ASE ratio estimate")}
    if (is.null(dim(fpost4)))
    {post4mode<-delta4[which.max(fpost4)]
     postinf4<-c(posat4,'ASE-L',post4mode)
     postden4<-c(posat4,fpost4)
     names(postden4)<-c("Locus of SNP", "ID",as.character(delta4))
     names(postinf4)<-c("Locus of SNP", "ID","ASE Status","ASE ratio estimate")}
    postinf<-rbind(postinf,postinf4)
  }
  
  rsobj<-list(allecounts=allecounts,GenoEsti=GenotypeEsti, Parameters=Parameters,Convergence=converge,niter=repcounts,ASEest=postinf, PostdenH=postden3, PostdenL=postden4)
  return(rsobj)
}