\name{select_run}
\alias{select_run}
\title{
select_run function
}
\description{
A function to run a particular SNP segment.
}
\usage{
select_run(gpr, members, tcon, epsilon0=0.0001, alpha10 = 3, 
           beta010 = 3, alpha20 = 3, beta020 = 3, estimate.alpha = TRUE, 
           common.alpha = TRUE, dist.dep = FALSE)
}
\arguments{
  \item{gpr}{
Group index to run
}
  \item{members}{
Membership information obtained from membershipc() or membership()
}
  \item{tcon}{
Original data set in data(total)
}
  \item{epsilon0}{
Error rate, a value between 0 and 1. 
}
  \item{alpha10}{
Starting values for the first parameter, alpha, in beta distribution for ASE-Low ratio.
}
  \item{beta010}{
Starting values for the second parameter, beta, in beta distribution for ASE-Low ratio.
}
  \item{alpha20}{
Starting values for the first parameter, alpha, in beta distribution for ASE-High ratio.
}
  \item{beta020}{
Starting values for the second parameter, beta, in beta distribution for ASE-High ratio.
}
  \item{estimate.alpha}{
TRUE or FALSE for whether or not estimating the alpha parameter in the beta distributions. 
By default estimate.alpha = TRUE. 
}
  \item{common.alpha}{
TRUE or FALSE. alpha1 is set to be the same as alpha2 if common.alpha=TRUE.  
}
  \item{dist.dep}{
TRUE or FALSE. If dist.dep = TRUE, the transition probability is modeled as a function of
distances among adjacent SNPs. By default, dist.dep = FALSE. 
}
}
\value{
A list of 8. 
\item{allecounts }{The original input data. }
\item{GenoEsti }{A 3 dimentional array with genotype prediction result [SNP, Animal, replicate].
                 For real data, #replicate=1, so the dimension is [#SNPs, #Animals, 1].
                 1 represents homogeneous genotype carrying two alleles form the first allele, 2 represents heterozygous genotype
                 with no ASE, 3 represents heterozygous genotype with first allele ASE, 4 represents 
                 heterozygous genotype with second allele ASE, 5 represents homogeneous genotype carrying two alleles form 
                 the second allele.}
\item{Parameters }{A vector of length 31 if dist.dep=TRUE. Estimation of paramter values 
                   after convergence. These parameters includes epsilon, alpha1, beta1, alpha2, beta2, rho, and a 5 by 5 
                   transition matrix in a vector form. It is of length 30 if dist.dep=FALSE with the parameter rho deleted.}
\item{Convergence }{Returns TRUE if convergence criterion is met. FALSE if convergence criterion                    
                    is not met. }
\item{niter }{Number of iterations take.}
\item{ASEest }{The ASE ratio estimate for the ASE-High and ASE-Low SNPs.}
\item{PostdenH }{The posterior densities of ASE ratio evaluated at a list of ASE ratio from 0.5 to 1, for the ASE-High SNPs.}
\item{PostdenL }{The posterior densities of ASE ratio evaluated at a list of ASE ratio from 0 to 0.5, for the ASE-Low SNPs.}
}
\references{
J. Steibel, H. Wang, P.S. Zhong (2013). A Hidden Markov Approach for Ascertaining SNP
Genotypes from Next Generation Sequencing Data in Presence of Allelic Imbalance.
}
\author{
J. Steibel, H. Wang, P.S. Zhong
}
\seealso{
See Also as \code{\link{HMM}}
}
\examples{
library(HMMASE)
library(R.utils)
data(total)
mbs<-membership(total,5000,2)
reps=1
epsilon0<-0.0001 
alpha10=3
beta010=3
alpha20=3
beta020=3
maxiter=20
estimate.alpha=TRUE
common.alpha=TRUE
dist.dep=FALSE

result<-select_run(gpr=5,members=mbs,tcon=total,epsilon0=epsilon0, 
alpha10=alpha10, beta010=beta010, alpha20=alpha20, beta020=beta020, 
estimate.alpha=estimate.alpha, common.alpha=common.alpha, 
dist.dep=dist.dep)
}

